package de.lluni.javann.util;

import com.opencsv.CSVReader;
import com.opencsv.exceptions.CsvValidationException;
import org.ejml.simple.SimpleMatrix;

import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

public class Utilities {
    private static final double STANDARD_GAUSSIAN_FACTOR = 1.0d;

    private static final double STANDARD_RANDOM_ORIGIN = -1.0d;
    private static final double STANDARD_RANDOM_BOUND = 1.0d;

    /**
     * Creates a matrix filled with ones
     * @param rows amount of rows
     * @param columns amount of columns
     * @return matrix filled with ones
     */
    public static SimpleMatrix ones(int rows, int columns) {
        SimpleMatrix mat = new SimpleMatrix(rows, columns);
        Arrays.fill(mat.getDDRM().data, 1);
        return mat;
    }

    /**
     * Creates a matrix with Gaussian distributed values
     * @param rows amount of rows
     * @param columns amount of columns
     * @param mean mean of Gaussian distribution
     * @param stddev standard deviation of Gaussian distribution
     * @param factor factor to multiply each Gaussian distributed value with
     * @return matrix of Gaussian distributed values
     */
    public static SimpleMatrix gaussianMatrix(int rows, int columns, double mean, double stddev, double factor) {
        SimpleMatrix mat = new SimpleMatrix(rows, columns);
        Random random = new Random();

        for (int i = 0; i < mat.getNumElements(); i++) {
            mat.set(i, factor * random.nextGaussian(mean, stddev));
        }

        return mat;
    }

    public static SimpleMatrix gaussianMatrix(int rows, int columns, double mean, double stddev) {
        return gaussianMatrix(rows, columns, mean, stddev, STANDARD_GAUSSIAN_FACTOR);
    }

    /**
     * Creates a matrix with random values
     * @param rows amount of rows
     * @param columns amount of columns
     * @param origin minimum random value
     * @param bound maximum random value
     * @return matrix with random values
     */
    public static SimpleMatrix randomMatrix(int rows, int columns, double origin, double bound) {
        Random random = new Random();
        return new SimpleMatrix(rows, columns, true,
                random.doubles((long) rows * columns, origin, bound).toArray());
    }

    public static SimpleMatrix randomMatrix(int rows, int columns) {
        return randomMatrix(rows, columns, STANDARD_RANDOM_ORIGIN, STANDARD_RANDOM_BOUND);
    }

    /**
     * Creates an array of evenly spaced values from the interval [start, end)
     * @param start start value
     * @param end end value
     * @param num amount of values
     * @return array of values
     */
    public static double[] linspace(double start, double end, int num) {
        double[] result = new double[num];
        double stepSize = Math.abs(end - start) / num;
        double nextEntry = start;

        for (int i = 0; i < num; i++) {
            result[i] = nextEntry;
            nextEntry += stepSize;
        }

        return result;
    }

    public static List<List<String>> readCSV(String filename) {
        List<List<String>> entries = new ArrayList<>();
        try (CSVReader csvReader = new CSVReader(new FileReader(filename))) {
            String[] values;
            while ((values = csvReader.readNext()) != null) {
                entries.add(Arrays.asList(values));
            }
            return entries;
        } catch (IOException e) {
            System.out.println(filename + " does not exist");
        } catch (CsvValidationException e) {
            System.out.println("Invalid line in " + filename);
        }
        return null;
    }
}
